@@ -9,7 +9,7 @@
  #  Note        : Include this file in your C++ source code, if you
  #                want to use the G'MIC interpreter in your own program.
  #
- #  Copyright   : David Tschumperle
+ #  Copyright   : David Tschumperlé
  #                ( https://tschumperle.users.greyc.fr/ )
  #
  #  Licenses    : This file is 'dual-licensed', you have to choose one
@@ -52,7 +52,7 @@
 */
 
 #ifndef gmic_version
-#define gmic_version 283
+#define gmic_version 300
 
 #ifndef gmic_pixel_type
 #define gmic_pixel_type float
@@ -61,6 +61,19 @@
 #include <cstdio>
 #include <cstring>
 
+// Define gmic_uint64 type.
+#ifndef gmic_uint64
+#if cimg_OS==2
+#define gmic_uint64 __int64
+#else // #if cimg_OS==2
+#if UINTPTR_MAX==0xffffffff || defined(__arm__) || defined(_M_ARM) || ((ULONG_MAX)==(UINT_MAX))
+#define gmic_uint64 unsigned long long
+#else
+#define gmic_uint64 unsigned long
+#endif // #if UINTPTR_MAX==0xffffffff || defined(__arm__) || defined(_M_ARM) || ((ULONG_MAX)==(UINT_MAX))
+#endif // #if cimg_OS==2
+#endif // #ifndef gmic_uint64
+
 #ifndef gmic_build
 
 // Define classes 'gmic_image<T>' and 'gmic_list<T>'.
@@ -167,12 +180,12 @@ namespace cimg_library {
 
 #ifndef cimg_verbosity
 #define cimg_verbosity 1
-#endif // #ifndef cimg_verbosity
+#endif
 
 #ifdef _MSC_VER
 #pragma comment(linker,"/STACK:6291456")
 #pragma inline_depth(2)
-#endif // #ifdef _MSC_VER
+#endif
 
 #include <locale>
 #ifdef cimg_version
@@ -183,26 +196,55 @@ namespace cimg_library {
 
 #ifdef cimg_use_abort
 inline bool *gmic_abort_ptr(bool *const p_is_abort);
-#define cimg_abort_init bool *const gmic_is_abort = ::gmic_abort_ptr(0)
+#define cimg_abort_init bool *const gmic_is_abort = ::gmic_abort_ptr(0); cimg::unused(gmic_is_abort)
 #define cimg_abort_test if (*gmic_is_abort) throw CImgAbortException()
-#endif // #ifdef cimg_use_abort
+#endif
 
 template<typename T>
-inline double gmic_mp_call(char *const str, void *const p_list, const T& pixel_type);
-#define cimg_mp_call_function(str) return ::gmic_mp_call(str,&mp.listout,(T)0)
+inline double gmic_mp_dollar(const char *const str,
+                             void *const p_list, const T& pixel_type);
+#define cimg_mp_operator_dollar(str) \
+  ::gmic_mp_dollar(str,&listout,(T)0)
 
 template<typename Ts, typename T>
-inline double gmic_mp_store(const Ts *const ptr,
+inline double gmic_mp_get(Ts *const ptr, const unsigned int siz, const bool to_string, const char *const str,
+                          void *const p_list, const T& pixel_type);
+#define cimg_mp_func_get(ptr,siz,to_string,str) \
+  return ::gmic_mp_get(ptr,siz,to_string,str,&mp.listout,(T)0)
+
+template<typename Ts, typename T>
+inline double gmic_mp_set(Ts *const ptr, const unsigned int siz, const char *const str,
+                          void *const p_list, const T& pixel_type);
+#define cimg_mp_func_set(ptr,siz,str) \
+  return ::gmic_mp_set(ptr,siz,str,&mp.listout,(T)0)
+
+template<typename T, typename Ts>
+inline double gmic_mp_name(const unsigned int ind, Ts *const out_str, const unsigned int siz,
+                           void *const p_list, const T& pixel_type);
+#define cimg_mp_func_name(ind,out_str,siz) \
+  return ::gmic_mp_name(ind,out_str,siz,&mp.listout,(T)0)
+
+template<typename T>
+inline double gmic_mp_run(char *const str,
+                          void *const p_list, const T& pixel_type);
+#define cimg_mp_func_run(str) \
+  return ::gmic_mp_run(str,&mp.listout,(T)0)
+
+template<typename Ts, typename T>
+inline double gmic_mp_store(const Ts *const ptr, const unsigned int siz,
                             const unsigned int w, const unsigned int h, const unsigned int d, const unsigned int s,
-                            const char *const str, void *const p_list, const T& pixel_type);
-#define cimg_mp_store_function(ptr,w,h,d,s,str) return ::gmic_mp_store(ptr,w,h,d,s,str,&mp.listout,(T)0)
+                            const bool is_compressed, const char *const str,
+                            void *const p_list, const T& pixel_type);
+#define cimg_mp_func_store(ptr,siz,w,h,d,s,is_compressed,str) \
+  return ::gmic_mp_store(ptr,siz,w,h,d,s,is_compressed,str,&mp.listout,(T)0)
+
 #ifndef cimg_display
 #define cimg_display 0
-#endif // #ifndef cimg_display
+#endif
 #ifndef cimg_appname
 #define cimg_appname "gmic"
-#endif // #ifndef cimg_appname
-#include "./CImg.h"
+#endif
+#include "CImg.h"
 
 #if cimg_OS==2
 #include <process.h>
@@ -213,6 +255,7 @@ inline double gmic_mp_store(const Ts *const ptr,
 #include <sys/resource.h>
 #include <sys/syscall.h>
 #include <signal.h>
+
 #endif // #if cimg_OS==2
 
 // Define some special character codes used for replacement in double quoted strings.
@@ -235,7 +278,7 @@ struct gmic {
   // Constructors.
   gmic();
 
-  template<typename T=gmic_pixel_type>
+  template<typename T>
   gmic(const char *const commands_line,
        const char *const custom_commands=0,
        const bool include_stdlib=true,
@@ -248,7 +291,7 @@ struct gmic {
        const bool include_stdlib=true, float *const p_progress=0, bool *const p_is_abort=0);
 
   // Run G'MIC pipeline on an already-constructed object.
-  template<typename T=gmic_pixel_type>
+  template<typename T>
   gmic& run(const char *const commands_line,
             float *const p_progress=0, bool *const p_is_abort=0,
             const T& pixel_type=(T)0);
@@ -267,16 +310,29 @@ struct gmic {
   template<typename T>
   static bool search_sorted(const char *const str, const T& list, const unsigned int length, unsigned int &out_ind);
   template<typename T>
-  static double mp_call(char *const str, void *const p_list, const T& pixel_type);
+  static double mp_dollar(const char *const str,
+                          void *const p_list, const T& pixel_type);
   template<typename Ts, typename T>
-  static double mp_store(const Ts *const ptr,
+  static double mp_get(Ts *const ptr, const unsigned int siz, const bool to_string, const char *const str,
+                       void *const p_list, const T& pixel_type);
+  template<typename Ts, typename T>
+  static double mp_set(Ts *const ptr, const unsigned int siz, const char *const str,
+                       void *const p_list, const T& pixel_type);
+  template<typename T, typename Ts>
+  static double mp_name(const unsigned int ind, Ts *const out_str, const unsigned int siz,
+                        void *const p_list, const T& pixel_type);
+  template<typename T>
+  static double mp_run(char *const str,
+                       void *const p_list, const T& pixel_type);
+  template<typename Ts, typename T>
+  static double mp_store(const Ts *const ptr, const unsigned int siz,
                          const unsigned int w, const unsigned int h, const unsigned int d, const unsigned int s,
-                         const char *const str, void *const p_list, const T& pixel_type);
+                         const bool is_compressed, const char *const str,
+                         void *const p_list, const T& pixel_type);
   static bool get_debug_info(const char *const s, unsigned int &line_number, unsigned int &file_number);
   static int _levenshtein(const char *const s, const char *const t,
                           gmic_image<int>& d, const int i, const int j);
   static int levenshtein(const char *const s, const char *const t);
-  static bool check_filename(const char *const filename);
   static unsigned int hashcode(const char *const str, const bool is_variable);
   static bool command_has_arguments(const char *const command);
   static const char* basename(const char *const str);
@@ -292,6 +348,9 @@ struct gmic {
              const char *const custom_commands, const bool include_stdlib,
              float *const p_progress, bool *const p_is_abort);
 
+  gmic_image<char> get_variable(const char *const name,
+                                const unsigned int *const variables_sizes=0,
+                                const gmic_list<char> *const images_names=0) const;
   const char *set_variable(const char *const name, const char *const value,
                            const char operation='=',
                            const unsigned int *const variables_sizes=0);
@@ -299,9 +358,11 @@ struct gmic {
                            const unsigned int *const variables_sizes=0);
 
   gmic& add_commands(const char *const data_commands, const char *const commands_file=0,
+                     const bool add_debug_info=false,
                      unsigned int *count_new=0, unsigned int *count_replaced=0,
                      bool *const is_entrypoint=0);
   gmic& add_commands(std::FILE *const file, const char *const filename=0,
+                     const bool add_debug_info=false,
                      unsigned int *count_new=0, unsigned int *count_replaced=0,
                      bool *const is_entrypoint=0);
 
@@ -310,6 +371,7 @@ struct gmic {
                                     const bool _is_debug=false) const;
   gmic_image<char> callstack2string(const gmic_image<unsigned int>* callstack_selection,
                                     const bool _is_debug=false) const;
+  void pop_callstack(const unsigned int callstack_size);
 
   gmic_image<unsigned int> selection2cimg(const char *const string, const unsigned int indice_max,
                                           const gmic_list<char>& names, const char *const command,
@@ -335,7 +397,7 @@ struct gmic {
   gmic_image<char> substitute_item(const char *const source,
                                    gmic_list<T>& images, gmic_list<char>& images_names,
                                    gmic_list<T>& parent_images, gmic_list<char>& parent_images_names,
-				   const unsigned int *const variables_sizes,
+                                   const unsigned int *const variables_sizes,
                                    const gmic_image<unsigned int> *const command_selection,
                                    const bool is_image_expr);
 
@@ -344,7 +406,7 @@ struct gmic {
 
   template<typename T>
   gmic& print(const gmic_list<T>& list, const gmic_image<unsigned int> *const callstack_selection,
-	      const char *format, ...);
+              const char *format, ...);
 
   template<typename T>
   gmic& warn(const gmic_list<T>& list, const gmic_image<unsigned int> *const callstack_selection,
@@ -353,7 +415,7 @@ struct gmic {
   template<typename T>
   gmic& error(const bool output_header, const gmic_list<T>& list,
               const gmic_image<unsigned int> *const callstack_selection,
-	      const char *const command, const char *format, ...);
+              const char *const command, const char *format, ...);
 
   template<typename T>
   bool check_cond(const char *const expr, gmic_list<T>& images, const char *const command);
@@ -425,9 +487,10 @@ struct gmic {
   gmic_image<char> status;
 
   float focale3d, light3d_x, light3d_y, light3d_z, specular_lightness3d, specular_shininess3d, _progress, *progress;
-  unsigned long reference_time;
-  unsigned int nb_dowhiles, nb_fordones, nb_repeatdones, nb_carriages, debug_filename, debug_line, cimg_exception_mode;
-  int verbosity,render3d, renderd3d;
+  gmic_uint64 reference_time;
+  unsigned int nb_dowhiles, nb_fordones, nb_repeatdones, nb_carriages_default, nb_carriages_stdout,
+    debug_filename, debug_line, cimg_exception_mode;
+  int verbosity,render3d, renderd3d, network_timeout;
   bool allow_entrypoint, is_change, is_debug, is_running, is_start, is_return, is_quit, is_double3d, is_debug_info,
     _is_abort, *is_abort, is_abort_thread;
   const char *starting_commands_line;
@@ -462,17 +525,47 @@ struct gmic_exception {
   }
 };
 
+inline bool *gmic_abort_ptr(bool *const p_is_abort) {
+  return gmic::abort_ptr(p_is_abort);
+}
+
 template<typename T>
-inline double gmic_mp_call(char *const str, void *const p_list, const T& pixel_type) {
-  return gmic::mp_call(str,p_list,pixel_type);
+inline double gmic_mp_dollar(const char *const str,
+                             void *const p_list, const T& pixel_type) {
+  return gmic::mp_dollar(str,p_list,pixel_type);
 }
+
 template<typename Ts, typename T>
-inline double gmic_mp_store(const Ts *const ptr,
+inline double gmic_mp_get(Ts *const ptr, const unsigned int siz, const bool to_string, const char *const str,
+                          void *const p_list, const T& pixel_type) {
+  return gmic::mp_get(ptr,siz,to_string,str,p_list,pixel_type);
+}
+
+template<typename Ts, typename T>
+inline double gmic_mp_set(Ts *const ptr, const unsigned int siz, const char *const str,
+                          void *const p_list, const T& pixel_type) {
+  return gmic::mp_set(ptr,siz,str,p_list,pixel_type);
+}
+
+template<typename T, typename Ts>
+inline double gmic_mp_name(const unsigned int ind, Ts *const out_str, const unsigned int siz,
+                           void *const p_list, const T& pixel_type) {
+  return gmic::mp_name(ind,out_str,siz,p_list,pixel_type);
+}
+
+template<typename T>
+inline double gmic_mp_run(char *const str,
+                          void *const p_list, const T& pixel_type) {
+  return gmic::mp_run(str,p_list,pixel_type);
+}
+
+template<typename Ts, typename T>
+inline double gmic_mp_store(const Ts *const ptr, const unsigned int siz,
                             const unsigned int w, const unsigned int h, const unsigned int d, const unsigned int s,
-                            const char *const str, void *const p_list, const T& pixel_type) {
-  return gmic::mp_store(ptr,w,h,d,s,str,p_list,pixel_type);
+                            const bool is_compressed, const char *const str,
+                            void *const p_list, const T& pixel_type) {
+  return gmic::mp_store(ptr,siz,w,h,d,s,is_compressed,str,p_list,pixel_type);
 }
-inline bool *gmic_abort_ptr(bool *const p_is_abort) { return gmic::abort_ptr(p_is_abort); }
 
 #endif // #ifndef gmic_version
 
